=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2011-02-24 18:16:42 +0000
+++ drizzled/cursor.cc	2011-03-07 11:59:53 +0000
@@ -24,9 +24,7 @@
 */
 
 #include <config.h>
-
 #include <fcntl.h>
-
 #include <drizzled/error.h>
 #include <drizzled/field/epoch.h>
 #include <drizzled/gettext.h>
@@ -35,7 +33,6 @@
 #include <drizzled/item/int.h>
 #include <drizzled/lock.h>
 #include <drizzled/message/table.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/optimizer/cost_vector.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/plugin/event_observer.h>
@@ -49,8 +46,7 @@
 
 using namespace std;
 
-namespace drizzled
-{
+namespace drizzled {
 
 /****************************************************************************
 ** General Cursor functions

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2011-02-22 21:46:00 +0000
+++ drizzled/cursor.h	2011-03-07 12:31:41 +0000
@@ -31,7 +31,6 @@
 #include <drizzled/message/table.h>
 #include <drizzled/sql_list.h>
 #include <drizzled/thr_lock.h>
-#include <drizzled/xid.h>
 
 #include <bitset>
 #include <algorithm>

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2011-03-03 09:48:30 +0000
+++ drizzled/drizzled.cc	2011-03-07 12:31:41 +0000
@@ -56,7 +56,6 @@
 #include <drizzled/message/cache.h>
 #include <drizzled/module/load_list.h>
 #include <drizzled/module/registry.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/plugin/error_message.h>
 #include <drizzled/plugin/event_observer.h>

=== modified file 'drizzled/dynamic_array.cc'
--- drizzled/dynamic_array.cc	2011-03-01 18:51:28 +0000
+++ drizzled/dynamic_array.cc	2011-03-06 18:20:11 +0000
@@ -16,17 +16,13 @@
 /* Handling of arrays that can grow dynamicly. */
 
 #include <config.h>
+#include <algorithm>
+#include <drizzled/dynamic_array.h>
 #include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/m_string.h>
-
-#include <algorithm>
 
 using namespace std;
 
-namespace drizzled
-{
-
-static bool allocate_dynamic(DYNAMIC_ARRAY *array, uint32_t max_elements);
+namespace drizzled {
 
 /*
   Initiate dynamic array
@@ -66,7 +62,7 @@
     init_alloc=alloc_increment;
     init_buffer= 0;
   }
-  array->elements=0;
+  array->set_size(0);
   array->max_element=init_alloc;
   array->alloc_increment=alloc_increment;
   array->size_of_element=element_size;
@@ -96,12 +92,12 @@
 static void insert_dynamic(DYNAMIC_ARRAY *array, void* element)
 {
   unsigned char* buffer;
-  if (array->elements == array->max_element)
+  if (array->size() == array->max_element)
     buffer= alloc_dynamic(array);
   else
   {
-    buffer= array->buffer+(array->elements * array->size_of_element);
-    array->elements++;
+    buffer= array->buffer+(array->size() * array->size_of_element);
+    array->set_size(array->size() + 1);
   }
   memcpy(buffer,element, array->size_of_element);
 }
@@ -131,7 +127,7 @@
 
 unsigned char *alloc_dynamic(DYNAMIC_ARRAY *array)
 {
-  if (array->elements == array->max_element)
+  if (array->size() == array->max_element)
   {
     char *new_ptr;
     if (array->buffer == (unsigned char *)(array + 1))
@@ -145,7 +141,7 @@
                                      array->size_of_element)))
         return 0;
       memcpy(new_ptr, array->buffer,
-             array->elements * array->size_of_element);
+             array->size() * array->size_of_element);
     }
     else if (!(new_ptr= (char*) realloc(array->buffer,
                                         (array->max_element+
@@ -155,7 +151,8 @@
     array->buffer= (unsigned char*) new_ptr;
     array->max_element+=array->alloc_increment;
   }
-  return array->buffer+(array->elements++ * array->size_of_element);
+  array->set_size(array->size() + 1);
+  return array->buffer + ((array->size() - 1) * array->size_of_element);
 }
 
 
@@ -173,115 +170,11 @@
 
 unsigned char *pop_dynamic(DYNAMIC_ARRAY *array)
 {
-  if (array->elements)
-    return array->buffer+(--array->elements * array->size_of_element);
-  return 0;
-}
-
-/*
-  Replace element in array with given element and index
-
-  SYNOPSIS
-    set_dynamic()
-      array
-      element	Element to be inserted
-      idx	Index where element is to be inserted
-
-  DESCRIPTION
-    set_dynamic() replaces element in array.
-    If idx > max_element insert new element. Allocate memory if needed.
-
-  RETURN VALUE
-    true	Idx was out of range and allocation of new memory failed
-    false	Ok
-*/
-
-bool set_dynamic(DYNAMIC_ARRAY *array, unsigned char* element, uint32_t idx)
-{
-  if (idx >= array->elements)
-  {
-    if (idx >= array->max_element && allocate_dynamic(array, idx))
-      return true;
-    memset(array->buffer+array->elements*array->size_of_element, 0,
-           (idx - array->elements)*array->size_of_element);
-    array->elements=idx+1;
-  }
-  memcpy(array->buffer+(idx * array->size_of_element),element,
-	 (size_t) array->size_of_element);
-  return false;
-}
-
-
-/*
-  Ensure that dynamic array has enough elements
-
-  SYNOPSIS
-    allocate_dynamic()
-    array
-    max_elements        Numbers of elements that is needed
-
-  NOTES
-   Any new allocated element are NOT initialized
-
-  RETURN VALUE
-    false	Ok
-    true	Allocation of new memory failed
-*/
-
-static bool allocate_dynamic(DYNAMIC_ARRAY *array, uint32_t max_elements)
-{
-  if (max_elements >= array->max_element)
-  {
-    uint32_t size;
-    unsigned char *new_ptr;
-    size= (max_elements + array->alloc_increment)/array->alloc_increment;
-    size*= array->alloc_increment;
-    if (array->buffer == (unsigned char *)(array + 1))
-    {
-       /*
-         In this senerio, the buffer is statically preallocated,
-         so we have to create an all-new malloc since we overflowed
-       */
-       if (!(new_ptr= (unsigned char *) malloc(size *
-                                               array->size_of_element)))
-         return 0;
-       memcpy(new_ptr, array->buffer,
-              array->elements * array->size_of_element);
-     }
-     else
-
-
-    if (!(new_ptr=(unsigned char*) realloc(array->buffer,
-                                        size* array->size_of_element)))
-      return true;
-    array->buffer= new_ptr;
-    array->max_element= size;
-  }
-  return false;
-}
-
-
-/*
-  Get an element from array by given index
-
-  SYNOPSIS
-    get_dynamic()
-      array
-      unsigned char*	Element to be returned. If idx > elements contain zeroes.
-      idx	Index of element wanted.
-*/
-
-void get_dynamic(DYNAMIC_ARRAY *array, unsigned char* element, uint32_t idx)
-{
-  if (idx >= array->elements)
-  {
-    memset(element, 0, array->size_of_element);
-    return;
-  }
-  memcpy(element,array->buffer+idx*array->size_of_element,
-         (size_t) array->size_of_element);
-}
-
+  if (!array->size())
+    return 0;
+  array->set_size(array->size() - 1);
+  return array->buffer+(array->size() * array->size_of_element);
+}
 
 /*
   Empty array by freeing all memory
@@ -297,13 +190,13 @@
     Just mark as empty if we are using a static buffer
   */
   if (array->buffer == (unsigned char *)(array + 1))
-    array->elements= 0;
+    array->set_size(0);
   else
   if (array->buffer)
   {
     free(array->buffer);
     array->buffer=0;
-    array->elements=array->max_element=0;
+    array->set_size(array->max_element=0);
   }
 }
 

=== modified file 'drizzled/dynamic_array.h'
--- drizzled/dynamic_array.h	2011-03-01 19:15:55 +0000
+++ drizzled/dynamic_array.h	2011-03-06 17:18:57 +0000
@@ -29,7 +29,6 @@
 {
 public:
   unsigned char *buffer;
-  size_t elements;
   size_t max_element;
   uint32_t alloc_increment;
   uint32_t size_of_element;
@@ -40,6 +39,13 @@
   {
     return elements;
   }
+
+  void set_size(size_t v)
+  {
+    elements = v;
+  }
+private:
+  size_t elements;
 };
 
 #define my_init_dynamic_array(A,B,C,D) init_dynamic_array2(A,B,NULL,C,D)
@@ -48,13 +54,8 @@
 bool init_dynamic_array2(DYNAMIC_ARRAY *array,uint32_t element_size,
                                    void *init_buffer, uint32_t init_alloc,
                                    uint32_t alloc_increment);
-/* init_dynamic_array() function is deprecated */
-bool init_dynamic_array(DYNAMIC_ARRAY *array,uint32_t element_size,
-                                  uint32_t init_alloc,uint32_t alloc_increment);
 unsigned char *alloc_dynamic(DYNAMIC_ARRAY *array);
 unsigned char *pop_dynamic(DYNAMIC_ARRAY*);
-bool set_dynamic(DYNAMIC_ARRAY *array,unsigned char * element,uint32_t array_index);
-void get_dynamic(DYNAMIC_ARRAY *array,unsigned char * element,uint32_t array_index);
 void delete_dynamic(DYNAMIC_ARRAY *array);
 
 } /* namespace drizzled */

=== modified file 'drizzled/ha_commands.cc'
--- drizzled/ha_commands.cc	2011-02-17 00:14:13 +0000
+++ drizzled/ha_commands.cc	2011-03-07 12:31:41 +0000
@@ -24,7 +24,6 @@
 */
 
 #include <config.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/error.h>
 #include <drizzled/gettext.h>
 #include <drizzled/probes.h>

=== modified file 'drizzled/internal/default.cc'
--- drizzled/internal/default.cc	2011-03-01 18:51:28 +0000
+++ drizzled/internal/default.cc	2011-03-06 18:20:11 +0000
@@ -41,7 +41,7 @@
 #include <drizzled/typelib.h>
 #include <drizzled/configmake.h>
 #include <drizzled/gettext.h>
-
+#include <drizzled/dynamic_array.h>
 #include <drizzled/cached_directory.h>
 
 #ifdef HAVE_SYS_STAT_H
@@ -448,13 +448,13 @@
     Here error contains <> 0 only if we have a fully specified conf_file
     or a forced default file
   */
-  if (!(ptr=(char*) alloc.alloc_root(sizeof(alloc)+ (args.elements + *argc +1) *sizeof(char*))))
+  if (!(ptr=(char*) alloc.alloc_root(sizeof(alloc)+ (args.size() + *argc +1) *sizeof(char*))))
     goto err;
   res= (char**) (ptr+sizeof(alloc));
 
   /* copy name + found arguments + command line arguments to new array */
   res[0]= argv[0][0];  /* Name MUST be set, even by embedded library */
-  memcpy(res+1, args.buffer, args.elements*sizeof(char*));
+  memcpy(res+1, args.buffer, args.size()*sizeof(char*));
   /* Skip --defaults-xxx options */
   (*argc)-= args_used;
   (*argv)+= args_used;
@@ -464,10 +464,10 @@
     This options must always be the last of the default options
   */
   if (*argc)
-    memcpy(res+1+args.elements, *argv + 1, (*argc-1)*sizeof(char*));
-  res[args.elements+ *argc]=0;			/* last null */
+    memcpy(res+1+args.size(), *argv + 1, (*argc-1)*sizeof(char*));
+  res[args.size()+ *argc]=0;			/* last null */
 
-  (*argc)+=int(args.elements);
+  (*argc)+=int(args.size());
   *argv= static_cast<char**>(res);
   *(memory::Root*) ptr= alloc;			/* Save alloc root for free */
   delete_dynamic(&args);

=== modified file 'drizzled/internal/my_sys.h'
--- drizzled/internal/my_sys.h	2011-02-22 06:09:35 +0000
+++ drizzled/internal/my_sys.h	2011-03-06 18:20:11 +0000
@@ -48,8 +48,6 @@
 #endif
 #endif					/* #ifndef errno */
 
-#include <drizzled/dynamic_array.h>
-
 #ifdef HAVE_SYS_MMAN_H 
 #include <sys/mman.h>
 #endif

=== modified file 'drizzled/item/subselect.cc'
--- drizzled/item/subselect.cc	2011-03-03 08:29:06 +0000
+++ drizzled/item/subselect.cc	2011-03-07 09:30:45 +0000
@@ -1208,6 +1208,7 @@
     select_lex->having= join->having= and_items(join->having, item);
     if (join->having == item)
       item->name= (char*)in_having_cond;
+    select_lex->having->top_level_item();
     select_lex->having_fix_field= 1;
     /*
       we do not check join->having->fixed, because Item_and (from and_items)

=== modified file 'drizzled/join.cc'
--- drizzled/join.cc	2011-03-01 01:45:32 +0000
+++ drizzled/join.cc	2011-03-07 11:59:53 +0000
@@ -63,13 +63,13 @@
 #include <drizzled/select_result.h>
 #include <drizzled/debug.h>
 #include <drizzled/item/subselect.h>
-
+#include <drizzled/my_hash.h>
 #include <algorithm>
 
 using namespace std;
 
-namespace drizzled
-{
+namespace drizzled {
+
 extern plugin::StorageEngine *heap_engine;
 
 /** Declarations of static functions used in this source file. */
@@ -823,7 +823,7 @@
       (select_options & SELECT_DESCRIBE) &&
       select_lex->master_unit() == &session->getLex()->unit) // upper level SELECT
   {
-    conds=new Item_int((int64_t) 0,1);  // Always false
+    conds=new Item_int(0, 1);  // Always false
   }
 
   if (make_join_select(this, select, conds))
@@ -1513,7 +1513,7 @@
     session->set_proc_info("Copying to tmp table");
     if (! curr_join->sort_and_group && curr_join->const_tables != curr_join->tables)
       curr_join->join_tab[curr_join->const_tables].sorted= 0;
-    if ((tmp_error= do_select(curr_join, (List<Item> *) 0, curr_tmp_table)))
+    if ((tmp_error= do_select(curr_join, NULL, curr_tmp_table)))
     {
       error= tmp_error;
       return;
@@ -1612,7 +1612,7 @@
               exec_tmp_table2= create_tmp_table(session,
                                                 &curr_join->tmp_table_param,
                                                 *curr_all_fields,
-                                                (Order*) 0,
+                                                NULL,
                                                 curr_join->select_distinct &&
                                                 !curr_join->group_list,
                                                 1, curr_join->select_options,
@@ -1664,7 +1664,7 @@
         curr_join->join_tab[curr_join->const_tables].sorted= 0;
       
       if (setup_sum_funcs(curr_join->session, curr_join->sum_funcs) 
-        || (tmp_error= do_select(curr_join, (List<Item> *) 0, curr_tmp_table)))
+        || (tmp_error= do_select(curr_join, NULL, curr_tmp_table)))
       {
         error= tmp_error;
         return;
@@ -2324,7 +2324,7 @@
           Item* new_item= new Item_func_rollup_const(item);
           if (!new_item)
             return 1;
-          new_item->fix_fields(session, (Item **) 0);
+          new_item->fix_fields(session, NULL);
           *it.ref()= new_item;
           for (Order *tmp= group_tmp; tmp; tmp= tmp->next)
           {
@@ -4727,7 +4727,7 @@
         COND *const_cond=
           make_cond_for_table(cond,
               join->const_table_map,
-              (table_map) 0, 1);
+              0, 1);
         for (JoinTable *tab= join->join_tab+join->const_tables;
             tab < join->join_tab+join->tables ; tab++)
         {
@@ -5750,7 +5750,7 @@
       if (!table->cursor->stats.records && !embedding)
       {						// Empty table
         s->dependent= 0;                        // Ignore LEFT JOIN depend.
-        set_position(join, const_count++, s, (optimizer::KeyUse*) 0);
+        set_position(join, const_count++, s, NULL);
         continue;
       }
       outer_join|= table->map;
@@ -5778,7 +5778,7 @@
 	      (table->cursor->getEngine()->check_flag(HTON_BIT_STATS_RECORDS_IS_EXACT)) &&
         !join->no_const_tables)
     {
-      set_position(join, const_count++, s, (optimizer::KeyUse*) 0);
+      set_position(join, const_count++, s, NULL);
     }
   }
   stat_vector[i]=0;
@@ -5902,7 +5902,7 @@
             table->mark_as_null_row();
             found_const_table_map|= table->map;
             join->const_table_map|= table->map;
-            set_position(join, const_count++, s, (optimizer::KeyUse*) 0);
+            set_position(join, const_count++, s, NULL);
             goto more_const_tables_found;
            }
           keyuse++;
@@ -5921,7 +5921,7 @@
           int tmp= 0;
           s->type= AM_SYSTEM;
           join->const_table_map|=table->map;
-          set_position(join, const_count++, s, (optimizer::KeyUse*) 0);
+          set_position(join, const_count++, s, NULL);
           partial_pos= join->getSpecificPosInPartialPlan(const_count - 1);
           if ((tmp= s->joinReadConstTable(partial_pos)))
           {
@@ -6069,7 +6069,7 @@
           caller to abort with a zero row result.
         */
         join->const_table_map|= s->table->map;
-        set_position(join, const_count++, s, (optimizer::KeyUse*) 0);
+        set_position(join, const_count++, s, NULL);
         s->type= AM_CONST;
         if (*s->on_expr_ref)
         {
@@ -6175,7 +6175,7 @@
 */
 static Table *get_sort_by_table(Order *a, Order *b,TableList *tables)
 {
-  table_map map= (table_map) 0;
+  table_map map= 0;
 
   if (!a)
     a= b;					// Only one need to be given

=== modified file 'drizzled/join.h'
--- drizzled/join.h	2011-02-23 00:13:15 +0000
+++ drizzled/join.h	2011-03-06 18:20:11 +0000
@@ -27,6 +27,7 @@
 #ifndef DRIZZLED_JOIN_H
 #define DRIZZLED_JOIN_H
 
+#include <drizzled/dynamic_array.h>
 #include <drizzled/optimizer/position.h>
 #include <drizzled/sql_select.h>
 #include <drizzled/tmp_table_param.h>

=== modified file 'drizzled/locking/global.cc'
--- drizzled/locking/global.cc	2011-02-24 13:19:47 +0000
+++ drizzled/locking/global.cc	2011-03-07 12:31:41 +0000
@@ -81,7 +81,6 @@
 #include <fcntl.h>
 
 #include <drizzled/error.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/thr_lock.h>
 #include <drizzled/session.h>
 #include <drizzled/sql_base.h>

=== modified file 'drizzled/my_hash.cc'
--- drizzled/my_hash.cc	2011-03-01 18:51:28 +0000
+++ drizzled/my_hash.cc	2011-03-07 11:59:53 +0000
@@ -25,9 +25,9 @@
 #include <drizzled/my_hash.h>
 #include <drizzled/charset.h>
 #include <drizzled/charset_info.h>
+#include <vector>
 
-namespace drizzled
-{
+namespace drizzled {
 
 const uint32_t NO_RECORD= UINT32_MAX;
 
@@ -36,22 +36,11 @@
 const int HIGHFIND= 4;
 const int HIGHUSED= 8;
 
-typedef struct st_hash_info {
-  /* index to next key */
-  uint32_t next;
-  /* data for current entry */
-  unsigned char *data;
-} HASH_LINK;
-
-static uint32_t hash_mask(uint32_t hashnr, uint32_t buffmax,
-                          uint32_t maxlength);
-static void movelink(HASH_LINK *array, uint32_t pos,
-                     uint32_t next_link, uint32_t newlink);
-static int hashcmp(const HASH *hash, HASH_LINK *pos, const unsigned char *key,
-                   size_t length);
-
-static uint32_t calc_hash(const HASH *hash, const unsigned char *key,
-                          size_t length)
+static uint32_t hash_mask(uint32_t hashnr, uint32_t buffmax, uint32_t maxlength);
+static void movelink(HASH_LINK *array, uint32_t pos, uint32_t next_link, uint32_t newlink);
+static int hashcmp(const HASH *hash, HASH_LINK *pos, const unsigned char *key, size_t length);
+
+static uint32_t calc_hash(const HASH *hash, const unsigned char *key, size_t length)
 {
   uint32_t nr1=1, nr2=4;
   hash->charset->coll->hash_sort(hash->charset, key,length, &nr1, &nr2);

=== modified file 'drizzled/my_hash.h'
--- drizzled/my_hash.h	2011-02-17 00:14:13 +0000
+++ drizzled/my_hash.h	2011-03-07 11:59:53 +0000
@@ -24,8 +24,7 @@
 
 #include <drizzled/dynamic_array.h>
 
-namespace drizzled
-{
+namespace drizzled {
 
 typedef struct charset_info_st CHARSET_INFO;
 
@@ -41,18 +40,29 @@
 typedef unsigned char *(*hash_get_key)(const unsigned char *,size_t*,bool);
 typedef void (*hash_free_key)(void *);
 
-typedef struct st_hash {
+struct HASH_LINK 
+{
+  /* index to next key */
+  uint32_t next;
+  /* data for current entry */
+  unsigned char *data;
+} ;
+
+struct HASH
+{
+  // typedef std::vector<HASH_LINK> array_t;
+  typedef DYNAMIC_ARRAY array_t;
   /* Length of key if const length */
   size_t key_offset,key_length;
   uint32_t blength;
   uint32_t records;
   uint32_t flags;
   /* Place for hash_keys */
-  DYNAMIC_ARRAY array;
+  array_t array;
   hash_get_key get_key;
   hash_free_key free;
   const CHARSET_INFO *charset;
-} HASH;
+};
 
 /* A search iterator state */
 typedef uint32_t HASH_SEARCH_STATE;

=== modified file 'drizzled/optimizer/quick_group_min_max_select.cc'
--- drizzled/optimizer/quick_group_min_max_select.cc	2011-02-24 18:16:42 +0000
+++ drizzled/optimizer/quick_group_min_max_select.cc	2011-03-06 17:50:05 +0000
@@ -224,16 +224,9 @@
   if (quick_prefix_select &&
       group_prefix_len < quick_prefix_select->max_used_key_length)
   {
-    DYNAMIC_ARRAY *arr= NULL;
-    uint32_t inx;
-
-    for (inx= 0, arr= &quick_prefix_select->ranges; inx < arr->elements; inx++)
-    {
-      optimizer::QuickRange *range= NULL;
-
-      get_dynamic(arr, (unsigned char*)&range, inx);
-      range->flag &= ~(NEAR_MIN | NEAR_MAX);
-    }
+    DYNAMIC_ARRAY& arr= quick_prefix_select->ranges;
+    for (size_t inx= 0; inx < arr.size(); inx++)
+      reinterpret_cast<optimizer::QuickRange**>(arr.buffer)[inx]->flag &= ~(NEAR_MIN | NEAR_MAX);
   }
 }
 

=== modified file 'drizzled/optimizer/quick_range_select.cc'
--- drizzled/optimizer/quick_range_select.cc	2011-03-01 17:16:41 +0000
+++ drizzled/optimizer/quick_range_select.cc	2011-03-06 16:34:02 +0000
@@ -217,7 +217,7 @@
 
 bool optimizer::QuickRangeSelect::unique_key_range() const
 {
-  if (ranges.elements == 1)
+  if (ranges.size() == 1)
   {
     optimizer::QuickRange *tmp= *((optimizer::QuickRange**)ranges.buffer);
     if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
@@ -259,7 +259,7 @@
   };
   error= cursor->multi_range_read_init(&seq_funcs,
                                        (void*) this,
-                                       ranges.elements,
+                                       ranges.size(),
                                        mrr_flags);
   return error;
 }
@@ -308,7 +308,7 @@
         return result;
     }
 
-    uint32_t count= ranges.elements - (cur_range - (optimizer::QuickRange**) ranges.buffer);
+    uint32_t count= ranges.size() - (cur_range - (optimizer::QuickRange**) ranges.buffer);
     if (count == 0)
     {
       /* Ranges have already been used up before. None is left for read. */
@@ -351,7 +351,7 @@
 {
   optimizer::QuickRange *res= NULL;
   uint32_t min= 0;
-  uint32_t max= ranges.elements - 1;
+  uint32_t max= ranges.size() - 1;
   uint32_t mid= (max + min) / 2;
 
   while (min != max)
@@ -453,7 +453,7 @@
     optimizer::QuickRangeSelect(*q)
 {
   optimizer::QuickRange **pr= (optimizer::QuickRange**) ranges.buffer;
-  optimizer::QuickRange **end_range= pr + ranges.elements;
+  optimizer::QuickRange **end_range= pr + ranges.size();
   for (; pr != end_range; pr++)
   {
     rev_ranges.push_back(*pr);

=== modified file 'drizzled/optimizer/quick_range_select.h'
--- drizzled/optimizer/quick_range_select.h	2011-02-17 10:09:00 +0000
+++ drizzled/optimizer/quick_range_select.h	2011-03-06 18:20:11 +0000
@@ -20,6 +20,7 @@
 #ifndef DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
 #define DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
 
+#include <drizzled/dynamic_array.h>
 #include <drizzled/optimizer/range.h>
 
 #include <boost/dynamic_bitset.hpp>

=== modified file 'drizzled/optimizer/range.cc'
--- drizzled/optimizer/range.cc	2011-03-01 18:51:28 +0000
+++ drizzled/optimizer/range.cc	2011-03-06 16:34:02 +0000
@@ -3341,10 +3341,10 @@
   optimizer::SEL_ARG *next= NULL;
   ulong use_count=key1->use_count;
 
-  if (key1->elements != 1)
+  if (key1->size() != 1)
   {
-    key2->use_count+=key1->elements-1; //psergey: why we don't count that key1 has n-k-p?
-    key2->increment_use_count((int) key1->elements-1);
+    key2->use_count+=key1->size()-1; //psergey: why we don't count that key1 has n-k-p?
+    key2->increment_use_count((int) key1->size()-1);
   }
   if (key1->type == optimizer::SEL_ARG::MAYBE_KEY)
   {
@@ -4354,7 +4354,7 @@
   quick->qr_traversal_ctx.first=  (optimizer::QuickRange**)quick->ranges.buffer;
   quick->qr_traversal_ctx.cur=    (optimizer::QuickRange**)quick->ranges.buffer;
   quick->qr_traversal_ctx.last=   quick->qr_traversal_ctx.cur +
-                                  quick->ranges.elements;
+                                  quick->ranges.size();
   return &quick->qr_traversal_ctx;
 }
 

=== modified file 'drizzled/optimizer/sel_arg.h'
--- drizzled/optimizer/sel_arg.h	2010-12-18 04:43:40 +0000
+++ drizzled/optimizer/sel_arg.h	2011-03-06 16:34:02 +0000
@@ -20,11 +20,8 @@
 #ifndef DRIZZLED_OPTIMIZER_SEL_ARG_H
 #define DRIZZLED_OPTIMIZER_SEL_ARG_H
 
-namespace drizzled
-{
-
-namespace optimizer
-{
+namespace drizzled {
+namespace optimizer {
 
 class RangeParameter;
 
@@ -260,6 +257,11 @@
       type(type_arg)
   {}
 
+  int size() const
+  {
+    return elements;
+  }
+
   inline bool is_same(SEL_ARG *arg)
   {
     if (type != arg->type || part != arg->part)

=== modified file 'drizzled/plugin.h'
--- drizzled/plugin.h	2011-02-17 00:14:13 +0000
+++ drizzled/plugin.h	2011-03-07 12:31:41 +0000
@@ -31,12 +31,10 @@
 
 #include <drizzled/lex_string.h>
 #include <drizzled/sys_var.h>
-#include <drizzled/xid.h>
 
 #include <drizzled/visibility.h>
 
-namespace drizzled
-{
+namespace drizzled {
 
 class Session;
 class Item;

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2011-02-16 21:10:25 +0000
+++ drizzled/plugin/storage_engine.h	2011-03-07 11:59:53 +0000
@@ -46,7 +46,7 @@
 class TableList;
 class Session;
 class Cursor;
-typedef struct st_hash HASH;
+struct HASH;
 
 class TableShare;
 typedef bool (stat_print_fn)(Session *session, const char *type, uint32_t type_len,

=== modified file 'drizzled/schema.cc'
--- drizzled/schema.cc	2011-02-17 10:09:00 +0000
+++ drizzled/schema.cc	2011-03-07 12:31:41 +0000
@@ -27,7 +27,6 @@
 
 #include <drizzled/error.h>
 #include <drizzled/gettext.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/internal/m_string.h>
 #include <drizzled/session.h>
 #include <drizzled/schema.h>

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2011-03-01 01:45:32 +0000
+++ drizzled/session.h	2011-03-07 11:59:53 +0000
@@ -42,7 +42,6 @@
 #include <drizzled/ha_data.h>
 #include <drizzled/identifier.h>
 #include <drizzled/lex_column.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/named_savepoint.h>
 #include <drizzled/open_tables_state.h>
 #include <drizzled/plugin.h>

=== modified file 'drizzled/session/transactions.h'
--- drizzled/session/transactions.h	2011-02-07 21:47:36 +0000
+++ drizzled/session/transactions.h	2011-03-07 12:31:41 +0000
@@ -22,14 +22,13 @@
 #define DRIZZLED_SESSION_TRANSACTIONS_H
 
 #include <deque>
+#include <drizzled/xid.h>
 
-namespace drizzled
-{
+namespace drizzled {
 
 class Session;
 
-namespace session
-{
+namespace session {
 
 /**
  * Structure used to manage "statement transactions" and
@@ -39,15 +38,9 @@
  * Storage engines will be registered here when they participate in
  * a transaction. No engine is registered more than once.
  */
-class Transactions {
+class Transactions 
+{
 public:
-  Transactions() :
-    savepoints(),
-    all(),
-    stmt(),
-    xid_state()
-  { }
-
   std::deque<NamedSavepoint> savepoints;
 
   /**

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2011-03-03 08:29:06 +0000
+++ drizzled/sql_parse.cc	2011-03-07 12:31:41 +0000
@@ -19,7 +19,6 @@
 
 #include <drizzled/item/num.h>
 #include <drizzled/abort_exception.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/error.h>
 #include <drizzled/nested_join.h>
 #include <drizzled/query_id.h>

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2011-03-03 08:29:06 +0000
+++ drizzled/sql_select.cc	2011-03-07 11:59:53 +0000
@@ -67,6 +67,7 @@
 #include <drizzled/sort_field.h>
 #include <drizzled/select_result.h>
 #include <drizzled/key.h>
+#include <drizzled/my_hash.h>
 
 using namespace std;
 
@@ -682,8 +683,8 @@
         save_pos++;
       }
       i= (uint32_t) (save_pos - (optimizer::KeyUse*) keyuse->buffer);
-      set_dynamic(keyuse, (unsigned char*) &key_end, i);
-      keyuse->elements= i;
+      reinterpret_cast<optimizer::KeyUse*>(keyuse->buffer)[i] = key_end;
+      keyuse->set_size(i);
     }
   }
   return false;

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2011-03-03 13:38:56 +0000
+++ drizzled/sql_table.cc	2011-03-07 12:31:41 +0000
@@ -22,7 +22,6 @@
 #include <drizzled/gettext.h>
 #include <drizzled/data_home.h>
 #include <drizzled/sql_parse.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/sql_lex.h>
 #include <drizzled/session.h>
 #include <drizzled/sql_base.h>

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2011-03-01 19:35:29 +0000
+++ drizzled/transaction_services.cc	2011-03-07 12:31:41 +0000
@@ -49,7 +49,6 @@
 
 #include <config.h>
 #include <drizzled/current_session.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/error.h>
 #include <drizzled/gettext.h>
 #include <drizzled/probes.h>

=== modified file 'drizzled/xid.cc'
--- drizzled/xid.cc	2011-02-17 00:14:13 +0000
+++ drizzled/xid.cc	2011-03-07 17:13:53 +0000
@@ -25,11 +25,10 @@
 #include <drizzled/charset.h>
 #include <drizzled/global_charset_info.h>
 #include <drizzled/charset_info.h>
-
+#include <boost/thread/locks.hpp>
 #include <boost/thread/mutex.hpp>
 
-namespace drizzled
-{
+namespace drizzled {
 
 bool XID::eq(XID *xid)
 {
@@ -55,13 +54,12 @@
 
 void XID::set(uint64_t xid)
 {
-  my_xid tmp;
   formatID= 1;
   set(DRIZZLE_XID_PREFIX_LEN, 0, DRIZZLE_XID_PREFIX);
-  memcpy(data+DRIZZLE_XID_PREFIX_LEN, &server_id, sizeof(server_id));
-  tmp= xid;
-  memcpy(data+DRIZZLE_XID_OFFSET, &tmp, sizeof(tmp));
-  gtrid_length=DRIZZLE_XID_GTRID_LEN;
+  memcpy(data + DRIZZLE_XID_PREFIX_LEN, &server_id, sizeof(server_id));
+  my_xid tmp= xid;
+  memcpy(data + DRIZZLE_XID_OFFSET, &tmp, sizeof(tmp));
+  gtrid_length= DRIZZLE_XID_GTRID_LEN;
 }
 
 void XID::set(long g, long b, const char *d)
@@ -85,7 +83,7 @@
 my_xid XID::quick_get_my_xid()
 {
   my_xid tmp;
-  memcpy(&tmp, data+DRIZZLE_XID_OFFSET, sizeof(tmp));
+  memcpy(&tmp, data + DRIZZLE_XID_OFFSET, sizeof(tmp));
   return tmp;
 }
 
@@ -97,104 +95,37 @@
     quick_get_my_xid() : 0;
 }
 
-uint32_t XID::length()
+uint32_t XID::length() const
 {
   return sizeof(formatID)+sizeof(gtrid_length)+sizeof(bqual_length)+
     gtrid_length+bqual_length;
 }
 
-unsigned char *XID::key()
+const unsigned char *XID::key() const
 {
   return (unsigned char *)&gtrid_length;
 }
 
-uint32_t XID::key_length()
+uint32_t XID::key_length() const
 {
   return sizeof(gtrid_length)+sizeof(bqual_length)+gtrid_length+bqual_length;
 }
 
-/***************************************************************************
-  Handling of XA id cacheing
-***************************************************************************/
-boost::mutex LOCK_xid_cache;
-HASH xid_cache;
-
-unsigned char *xid_get_hash_key(const unsigned char *, size_t *, bool);
-void xid_free_hash(void *);
-
-unsigned char *xid_get_hash_key(const unsigned char *ptr, size_t *length,
-                        bool )
-{
-  *length=((XID_STATE*)ptr)->xid.key_length();
-  return ((XID_STATE*)ptr)->xid.key();
-}
-
-void xid_free_hash(void *ptr)
-{
-  XID_STATE *state= (XID_STATE *)ptr;
-  if (state->in_session == false)
-    delete state;
-}
-
 bool xid_cache_init()
 {
-  return hash_init(&xid_cache, &my_charset_bin, 100, 0, 0,
-                   xid_get_hash_key, xid_free_hash, 0) != 0;
+  return false;
 }
 
 void xid_cache_free()
 {
-  if (hash_inited(&xid_cache))
-  {
-    hash_free(&xid_cache);
-  }
-}
-
-XID_STATE *xid_cache_search(XID *xid)
-{
-  LOCK_xid_cache.lock();
-  XID_STATE *res=(XID_STATE *)hash_search(&xid_cache, xid->key(), xid->key_length());
-  LOCK_xid_cache.unlock();
-  return res;
-}
-
-bool xid_cache_insert(XID *xid, enum xa_states xa_state)
-{
-  XID_STATE *xs;
-  bool res;
-  LOCK_xid_cache.lock();
-  if (hash_search(&xid_cache, xid->key(), xid->key_length()))
-  {
-    res= false;
-  }
-  else if ((xs = new XID_STATE) == NULL)
-  {
-    res= true;
-  }
-  else
-  {
-    xs->xa_state=xa_state;
-    xs->xid.set(xid);
-    xs->in_session=0;
-    res= my_hash_insert(&xid_cache, (unsigned char*)xs);
-  }
-  LOCK_xid_cache.unlock();
-  return res;
-}
-
-bool xid_cache_insert(XID_STATE *xid_state)
-{
-  LOCK_xid_cache.lock();
-  bool res=my_hash_insert(&xid_cache, (unsigned char*)xid_state);
-  LOCK_xid_cache.unlock();
-  return res;
-}
-
-void xid_cache_delete(XID_STATE *xid_state)
-{
-  LOCK_xid_cache.lock();
-  hash_delete(&xid_cache, (unsigned char *)xid_state);
-  LOCK_xid_cache.unlock();
+}
+
+void xid_cache_insert(XID*, xa_states)
+{
+}
+
+void xid_cache_delete(XID_STATE*)
+{
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/xid.h'
--- drizzled/xid.h	2011-02-13 01:25:00 +0000
+++ drizzled/xid.h	2011-03-07 16:59:16 +0000
@@ -22,8 +22,7 @@
 
 #include <cstring>
 
-namespace drizzled
-{
+namespace drizzled {
 
 extern uint32_t server_id;
 
@@ -43,10 +42,9 @@
 #define DRIZZLE_XID_OFFSET (DRIZZLE_XID_PREFIX_LEN+sizeof(server_id))
 #define DRIZZLE_XID_GTRID_LEN (DRIZZLE_XID_OFFSET+sizeof(my_xid))
 
-class XID {
-
+class XID 
+{
 public:
-
   long formatID;
   long gtrid_length;
   long bqual_length;
@@ -69,9 +67,9 @@
   void null();
   my_xid quick_get_my_xid();
   my_xid get_my_xid();
-  uint32_t length();
-  unsigned char *key();
-  uint32_t key_length();
+  uint32_t length() const;
+  const unsigned char* key() const;
+  uint32_t key_length() const;
 };
 
 /**
@@ -82,7 +80,8 @@
 
 */
 
-class DrizzleXid {
+class DrizzleXid 
+{
 public:
   long formatID;
   long gtrid_length;
@@ -109,20 +108,18 @@
 {
 public:
   XID_STATE() :
-    xid(),
     xa_state(XA_NOTR),
     in_session(false)
   {}
   /* For now, this is only used to catch duplicated external xids */
   XID  xid;                           // transaction identifier
-  enum xa_states xa_state;            // used by external XA only
+  xa_states xa_state;            // used by external XA only
   bool in_session;
 };
 
-bool xid_cache_init(void);
-void xid_cache_free(void);
-XID_STATE *xid_cache_search(XID *xid);
-bool xid_cache_insert(XID *xid, enum xa_states xa_state);
+bool xid_cache_init();
+void xid_cache_free();
+void xid_cache_insert(XID *xid, xa_states xa_state);
 bool xid_cache_insert(XID_STATE *xid_state);
 void xid_cache_delete(XID_STATE *xid_state);
 

=== modified file 'plugin/archive/archive_engine.h'
--- plugin/archive/archive_engine.h	2011-02-17 00:14:13 +0000
+++ plugin/archive/archive_engine.h	2011-03-07 11:59:53 +0000
@@ -20,7 +20,6 @@
 #include <drizzled/table.h>
 #include <drizzled/session.h>
 #include <drizzled/thr_lock.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/cursor.h>
 #include <drizzled/plugin/storage_engine.h>
 

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2011-03-02 09:37:25 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2011-03-07 12:31:41 +0000
@@ -48,7 +48,6 @@
 #include <drizzled/charset_info.h>
 #include <drizzled/internal/m_string.h>
 #include <drizzled/internal/my_sys.h>
-#include <drizzled/my_hash.h>
 #include <drizzled/plugin.h>
 #include <drizzled/show.h>
 #include <drizzled/data_home.h>

=== modified file 'plugin/myisam/myisam_priv.h'
--- plugin/myisam/myisam_priv.h	2011-02-17 00:14:13 +0000
+++ plugin/myisam/myisam_priv.h	2011-03-06 18:20:11 +0000
@@ -27,6 +27,7 @@
 #include <drizzled/thr_lock.h>
 #include <drizzled/common.h>
 #include <drizzled/enum.h>
+#include <drizzled/dynamic_array.h>
 
 #include <assert.h>
 #include <fcntl.h>

=== modified file 'plugin/myisam/sort.cc'
--- plugin/myisam/sort.cc	2011-03-01 18:19:16 +0000
+++ plugin/myisam/sort.cc	2011-03-06 16:34:02 +0000
@@ -288,12 +288,12 @@
   }
   if (error > 0)
     return(HA_POS_ERROR);
-  if (buffpek->elements)
+  if (buffpek->size())
   {
     if (info->write_keys(info,sort_keys,idx,(BUFFPEK *)alloc_dynamic(buffpek),
 		   tempfile))
       return(HA_POS_ERROR);
-    *maxbuffer=buffpek->elements-1;
+    *maxbuffer=buffpek->size() - 1;
   }
   else
     *maxbuffer=0;
@@ -330,7 +330,7 @@
     if (!got_error)
     {
       mi_set_key_active(share->state.key_map, sinfo->key);
-      if (!sinfo->buffpek.elements)
+      if (!sinfo->buffpek.size())
       {
         if (param->testflag & T_VERBOSE)
         {
@@ -375,9 +375,9 @@
       sinfo->read_to_buffer=read_to_buffer;
       sinfo->write_key=write_merge_key;
     }
-    if (sinfo->buffpek.elements)
+    if (sinfo->buffpek.size())
     {
-      size_t maxbuffer=sinfo->buffpek.elements-1;
+      size_t maxbuffer=sinfo->buffpek.size() - 1;
       if (!mergebuf)
       {
         length=param->sort_buffer_length;

=== modified file 'plugin/show_dictionary/show_table_status.cc'
--- plugin/show_dictionary/show_table_status.cc	2011-02-24 13:19:47 +0000
+++ plugin/show_dictionary/show_table_status.cc	2011-03-07 12:31:41 +0000
@@ -22,7 +22,6 @@
 
 #include <plugin/show_dictionary/dictionary.h>
 #include <drizzled/pthread_globals.h>
-#include <drizzled/my_hash.h>
 
 using namespace drizzled;
 using namespace std;

=== modified file 'plugin/table_cache_dictionary/table_cache.cc'
--- plugin/table_cache_dictionary/table_cache.cc	2011-02-24 13:19:47 +0000
+++ plugin/table_cache_dictionary/table_cache.cc	2011-03-07 12:31:41 +0000
@@ -22,7 +22,6 @@
 
 #include <plugin/table_cache_dictionary/dictionary.h>
 #include <drizzled/pthread_globals.h>
-#include <drizzled/my_hash.h>
 
 using namespace drizzled;
 using namespace std;

=== added file 'tests/suite/regression/r/709500.result'
--- tests/suite/regression/r/709500.result	1970-01-01 00:00:00 +0000
+++ tests/suite/regression/r/709500.result	2011-01-31 17:06:18 +0000
@@ -0,0 +1,61 @@
+drop table if exists t1,t1s,t2s;
+create table t1(i int);
+insert into t1 values (1), (2), (3);
+create table t1s(i int);
+insert into t1s values (10), (20), (30);
+create table t2s(i int);
+insert into t2s values (100), (200), (300);
+select straight_join t2s.i
+from
+t1s left outer join t2s on t2s.i = t1s.i;
+i
+NULL
+NULL
+NULL
+select straight_join t2s.i
+from
+t1s left outer join t2s on t2s.i = t1s.i
+having t2s.i = 999;
+i
+select * from t1
+where t1.i NOT IN
+(
+select straight_join t2s.i 
+from
+t1s left outer join t2s on t2s.i = t1s.i
+having t2s.i = 100
+);
+i
+1
+2
+3
+select * from t1
+where t1.i IN
+(
+select straight_join t2s.i 
+from
+t1s left outer join t2s on t2s.i = t1s.i
+having t2s.i = 100
+) IS UNKNOWN;
+i
+select * from t1
+where not t1.i = ANY
+(
+select straight_join t2s.i 
+from
+t1s left outer join t2s on t2s.i = t1s.i
+having t2s.i = 999
+);
+i
+1
+2
+3
+select * from t1
+where t1.i = ANY (
+select straight_join t2s.i 
+from
+t1s left outer join t2s on t2s.i = t1s.i
+having t2s.i = 999
+) is unknown;
+i
+drop table t1,t1s,t2s;

=== added file 'tests/suite/regression/t/709500.test'
--- tests/suite/regression/t/709500.test	1970-01-01 00:00:00 +0000
+++ tests/suite/regression/t/709500.test	2011-01-31 17:06:18 +0000
@@ -0,0 +1,57 @@
+--disable_warnings
+drop table if exists t1,t1s,t2s;
+--enable_warnings
+
+create table t1(i int);
+insert into t1 values (1), (2), (3);
+
+create table t1s(i int);
+insert into t1s values (10), (20), (30);
+create table t2s(i int);
+insert into t2s values (100), (200), (300);
+
+select straight_join t2s.i
+   from
+   t1s left outer join t2s on t2s.i = t1s.i;
+
+select straight_join t2s.i
+   from
+   t1s left outer join t2s on t2s.i = t1s.i
+   having t2s.i = 999;
+
+select * from t1
+where t1.i NOT IN
+(
+  select straight_join t2s.i 
+  from
+  t1s left outer join t2s on t2s.i = t1s.i
+  having t2s.i = 100
+);
+
+select * from t1
+where t1.i IN
+(
+  select straight_join t2s.i 
+  from
+  t1s left outer join t2s on t2s.i = t1s.i
+  having t2s.i = 100
+) IS UNKNOWN;
+
+select * from t1
+where not t1.i = ANY
+(
+  select straight_join t2s.i 
+  from
+  t1s left outer join t2s on t2s.i = t1s.i
+  having t2s.i = 999
+);
+
+select * from t1
+ where t1.i = ANY (
+  select straight_join t2s.i 
+  from
+  t1s left outer join t2s on t2s.i = t1s.i
+  having t2s.i = 999
+ ) is unknown;
+
+drop table t1,t1s,t2s;

